DailyItem_ADDONNAME = "DailyItems";
DailyItem_MacroNAME = "DailyItem";
DailyItem_VERSION = GetAddOnMetadata("DailyItems", "Version");

SLASH_DAILYITEM1, SLASH_DAILYITEM2 = '/dailyitem', '/di';

local DI_Debug = false;
--local intReleaseVer = tonumber(strsub(select(4, GetBuildInfo()),1,1)) or 1
local intReleaseVer = tonumber(select(4, GetBuildInfo())/10000) or 1

function SlashCmdList.DAILYITEM(msg)
	if (strlower(msg) == "zone") then
		DailyItem_Print("Zone = ["..GetRealZoneText().."]");
		DailyItem_Print("SubZone = ["..GetSubZoneText().."]");
		if DI_Debug then
			DailyItem_Print("mapID = [" .. C_Map.GetBestMapForUnit("player") .. "]");
		end
	elseif (strlower(msg) == "quests") then
		if intReleaseVer > 8 then
			DailyItem_ListQuests()
		else
			DailyItem_ListQuests_old()
		end
	elseif (strlower(msg) == "debug") then
		if DI_Debug then 
			DI_Debug = false;
			DailyItem_Print("Debug Off");
		else
			DI_Debug = true;
			DailyItem_Print("Debug On");
		end
	else
		print ("Daily items version ",DailyItem_VERSION)
		DailyItem_Print("Active entries - "..(table.getn(DailyItem_Quests)));
		print ("Daily item slash commands are:");
		print ("Zone - Show the zone and subzone you are in");
		print ("Quests - List the quests currently in your log");
	end
end

-- I want this to run when you log in, update bag contents, swap zones, change targets or update the quest log
function DailyItem_OnLoad(self)
	self:RegisterEvent("PLAYER_LOGIN");
	self:RegisterEvent("BAG_UPDATE");
	self:RegisterEvent("ZONE_CHANGED");
	self:RegisterEvent("PLAYER_TARGET_CHANGED");
	self:RegisterEvent("UNIT_QUEST_LOG_CHANGED");
end

function DailyItem_OnEvent(self, event)
if DI_Debug then DailyItem_Print("Start ("..event..")"); end
	if (InCombatLockdown()) then 
if DI_Debug then DailyItem_Print("Combat"); end
		return; 
	end
	if (event == "PLAYER_LOGIN") then
		local numGenMacros = GetNumMacros();
		if (GetMacroIndexByName(DailyItem_MacroNAME) == 0) then -- If we can't find the macro, create it.
			if (numGenMacros < 36) then
				CreateMacro(DailyItem_MacroNAME, "INV_MISC_QUESTIONMARK" , "", nil)
			else
				DailyItem_Print("You have 36 General Macros. Cannot add "..DailyItem_MacroNAME.."!");
			end	
		end
		DailyItem_MakeArray();
	elseif (event == "UNIT_QUEST_LOG_CHANGED") then -- Don't think we need to create macro at quest change.  
		DailyItem_MakeArray();
	else
		if DailyItem_Quests ~= nil and DailyItem_Quests ~= {} then
			local MacroText;
			local Icon = "INV_MISC_QUESTIONMARK";

			-- Process the array in the so that the LAST one found is the one we will keep
			DI_QuestsLength = #DailyItem_Quests;
			for Index = 1, DI_QuestsLength -- Cycle through all quests in the array
			do

			-- Better names to make it clearer
				DI_Zone = DailyItem_Quests[Index][1]
				DI_SubZone = DailyItem_Quests[Index][2]
				DI_Amount = DailyItem_Quests[Index][3]
				DI_Item = DailyItem_Quests[Index][4]
				DI_Target = DailyItem_Quests[Index][5]
				DI_Aim = DailyItem_Quests[Index][6]
				DI_Quest = DailyItem_Quests[Index][7]

				-- Check if we're in the right part of the world
				if DailyItem_IsZone(Index) and DailyItem_IsSubZone(Index) then 

-- if DI_Debug then DailyItem_Print("Found Zone"); end
					-- Check if there is an associated item to use
					if not IsEmpty(DI_Item) then

if DI_Debug then DailyItem_Print("Item ("..DI_Item..")"); end
						-- Do we have (enough of) it in our bags or worn
						if GetItemCount(DI_Item) >= DI_Amount or DailyItem_IsItemWorn(DI_Item) then 

if DI_Debug then DailyItem_Print("Count "..GetItemCount(DI_Item).."("..DI_Item..")"); end
							-- it should be equipped, but it's not (must be in bag)
							if IsEquippableItem(DI_Item) and not DailyItem_IsItemWorn(DI_Item) then 

if DI_Debug then DailyItem_Print("Equip ("..DI_Item..")"); end
								-- So equip it FFS
								MacroText = "#showtooltip\n/equip item:" .. DI_Item;
							else

								-- Do we need to target the NPC ourselves...?
								if DI_Aim and DI_Target then 

if DI_Debug then DailyItem_Print("Aiming for " .. DI_Target); end

									-- Create a table with all of the targets
									local DI_Targets = { strsplit(",", DI_Target) }

									-- Create a target line for each target given
									local DI_TargetMacro = ""
									table.foreach(
										DI_Targets, 
										function(k,v) 
											if DI_Aim == 1 then 
												DI_TargetMacro = DI_TargetMacro .. "/targetexact [noexists][dead]" .. v .. "\n" ;
											elseif DI_Aim == 2 then 
												DI_TargetMacro = DI_TargetMacro .. "/target [noexists][dead]" .. v .. "\n" ;
											elseif DI_Aim == 4 then 
												DI_TargetMacro = DI_TargetMacro .. "/target [noexists][nodead]" .. v .. "\n/cleartarget [nodead]\n" ;
											end
											if DI_Debug then 
												DailyItem_Print("Target ("..v..")"); 
											end 
										end
									) 

									-- It's over there....
									MacroText = "#showtooltip\n" .. DI_TargetMacro .."/use item:" .. DI_Item;

								 -- Do we have the right target already (or there is no specific target)?
								elseif IsEmpty(DI_Target) or StringMatch(DI_Target, UnitName("target")) then

									-- Use the item NOW dammit!!!
									MacroText = "#showtooltip\n/use item:" .. DI_Item;
								
								-- End of targeting section
								end

							-- End of equip or use item
							end

						-- We don't have the necessary item(s)
						end

					-- There is no item for this quest, so just target
					else
if DI_Debug then DailyItem_Print("No Item"); end
						-- Set an appropriate icon for the macro
						Icon = "ABILITY_HUNTER_SNIPERSHOT"
						MacroText = "#showtooltip\n/target [noexists][dead] " .. DI_Target

					-- End of item check
					end

				-- End of zone check
				end

			-- End of loop through each entry in the array
			end

			-- So, we've decided what we want in our Macro, so write it out (only if there's something to write)
			if MacroText  then

				-- Update the macro
				EditMacro(DailyItem_MacroNAME, nil, Icon , MacroText, nil, nil);

			-- Done writing out Macro
			end 

		-- Only run if the array exists
		end 
	end
end

--
-- Functions
-- 

function DailyItem_AddArrayLine(Zone,SubZone,Amount,Item,Target,Aim,Quest)
--DailyItem_Print('Line');
	if DailyItem_IsMatchQuest(Quest) then
		local temp = table.getn(DailyItem_Quests) + 1;
		DailyItem_Quests[temp] = {Zone,SubZone,Amount,Item,Target,Aim};
	end
end

function DailyItem_IsMatchQuest(Index)
	if IsEmpty(Index) then -- If there's no quest match requirement then always return true
		return true;
	end

	if intReleaseVer > 8 then
		numEntries, numQuests = C_QuestLog.GetNumQuestLogEntries(); -- Get the number of quests in my quest log
	else 
		numEntries, numQuests = GetNumQuestLogEntries(); -- Get the number of quests in my quest log
	end

	for questNo = 1, numEntries -- Now, cycle through each quest in the quest log
	do
		if intReleaseVer > 8 then -- SL and later
			local arrQuest = C_QuestLog.GetInfo(questNo);					-- Get all quest info
			if (arrQuest.questID > 0) then 									-- Ignore the lines that are just headers rather than actual quests
				local isComplete = C_QuestLog.IsComplete(arrQuest.questID)	-- It is complete
if DI_Debug then DailyItem_Print("Checking Quest " .. arrQuest.title .. " (" .. arrQuest.questID .. ")"); end
				if Index == arrQuest.questID and not isComplete then 		-- If we find the quest and it's not completed
					return true;
				end
				if Index == -arrQuest.questID then 							-- Or if we find a quest we didn't want to find
					return false;
				end
			end
		else
			local questTitle , _ , _ , isHeader, _ , isComplete, _, questID = GetQuestLogTitle(questNo); -- Get the information about each line in the quest log
if DI_Debug then DailyItem_Print("Checking Quest " .. questTitle .. " (" .. questID .. ")"); end
			if (questID > 0) then -- Ignore the lines that are just headers rather than actual quests
				if Index == questID and not isComplete then -- If we find the quest and it's not completed
					return true;
				end
				if Index == -questID then -- Or if we find a quest we didn't want to find
					return false;
				end
			end
		end
	end
	return false;
end

function DailyItem_ListQuests()
	local numEntries, numQuests = C_QuestLog.GetNumQuestLogEntries(); 	-- Get the number of quests in my quest log
	for questNo = 1, numEntries 										-- Now, cycle through each quest in the quest log
	do
		local arrQuest = C_QuestLog.GetInfo(questNo);					-- Get all quest info
		if (arrQuest.questID > 0 and not arrQuest.isHidden) then 									-- Ignore the lines that are just headers rather than actual quests
			DailyItem_Print(questNo .. " - " .. arrQuest.title .. " (" .. arrQuest.questID .. ")");
		end
	end
end

function DailyItem_ListQuests_old()
	local numEntries, numQuests = GetNumQuestLogEntries(); -- Get the number of quests in my quest log
	DailyItem_Print(numQuests .. " Quests listed:");
	for questNo = 1, numEntries -- Now, cycle through each quest in the quest log
	do
		local questTitle , _ , _ , isHeader, _ , isComplete, _, questID = GetQuestLogTitle(questNo); -- Get the information about each line in the quest log
		if (questID > 0) then -- Ignore the lines that are just headers rather than actual quests
			DailyItem_Print(questTitle .. " (" .. questID .. ")");
		end
	end
end


function DailyItem_IsItemWorn(itemID)
	for equipID = 0, 19, 1 do
		local itemNumber = GetInventoryItemID("player",equipID);
		if (itemNumber == itemID) then
			return true;
		end
	end
	return false;
end


function DailyItem_IsSubZone(Index)
	if intReleaseVer > 8 then
		if IsEmpty(DailyItem_Quests[Index][2]) then
if DI_Debug then DailyItem_Print("No SubZone"); end
			return true;
		else 
			for k, v in pairs({strsplit(",", DailyItem_Quests[Index][2])}) do
				if v == GetSubZoneText() then
if DI_Debug then DailyItem_Print("SubZone = " .. GetSubZoneText()); end
					return true;
				else
--if DI_Debug then DailyItem_Print("SubZone ~= " .. v); end
				end
			end
			return false;
		end
	else
		if IsEmpty(DailyItem_Quests[Index][2]) then
if DI_Debug then DailyItem_Print("No SubZone"); end
			return true;
		elseif DailyItem_Quests[Index][2] == GetSubZoneText() then
if DI_Debug then DailyItem_Print("SubZone = "..GetSubZoneText()); end
			return true;
		else 
--if DI_Debug then DailyItem_Print("SubZone ~= "..DailyItem_Quests[Index][2]); end
			return false;
		end
	end
end


function DailyItem_IsZone(Index)
	if IsEmpty(DailyItem_Quests[Index][1]) then
if DI_Debug then DailyItem_Print("No Zone"); end
		return true;
	elseif DailyItem_Quests[Index][1] == GetRealZoneText() then
if DI_Debug then DailyItem_Print("Zone = "..GetRealZoneText()); end
		return true;
	else
--if DI_Debug then DailyItem_Print("Zone ~= "..DailyItem_Quests[Index][1]); end
		return false;
	end
end

function DailyItem_NumFromLink(itemLink)
	if IsEmpty(itemLink) then
		return nil;
	end
	local _, _, itemNumber = strfind(itemLink, "item:(%d+):");
	return itemNumber;
end

function DailyItem_Print(msg)
	if ((msg) and (strlen(msg) > 0)) then
		if (DEFAULT_CHAT_FRAME) then
			DEFAULT_CHAT_FRAME:AddMessage("Daily Items - "..msg, 1, 1, 0);
		end
	end
end

function StringMatch(str1, str2)
	local s1, s2
	s1=str1
	s2=str2
	if s2 and s1 then
		if string.sub(s2,-1)=="*" then
			s2=string.sub(s2,1,string.len(s2)-1)
			s1=string.sub(s1,1,string.len(s2))
		end
		if string.sub(s1,-1)=="*" then
			s1=string.sub(s1,1,string.len(s1)-1)
			s2=string.sub(s2,1,string.len(s1))
		end
	end
	return s1 == s2
end

function IsEmpty(s)
  return s == nil or s == '' or s == {} or s == 0
end